import { Country } from '@/classes/Enum/Country'

export class IbanGenerator {
  generate (country: Country) {
    const shortBankAccount = this.getBankAccount(country)
    const bankCode = this.getBankCode(country)
    const estonianCheckDigit = this.getEstonianCheckDigit(bankCode + shortBankAccount)
    switch (country) {
      case Country.ESTONIA:
        return country.toUpperCase() + this.getIbanCheckDigits(bankCode + shortBankAccount + estonianCheckDigit + '141400') + bankCode + shortBankAccount + estonianCheckDigit
      case Country.LITHUANIA:
        return country.toUpperCase() + this.getIbanCheckDigits(bankCode + shortBankAccount + '212900') + bankCode + shortBankAccount
      default:
        return country.toUpperCase() + this.getIbanCheckDigits(bankCode + shortBankAccount + '252100') + bankCode + shortBankAccount
    }
  }

  getBankAccount (country: Country) {
    let t
    const i = BigInt(97)
    const o = BigInt((t = Country.POLAND === country
      ? Math.floor(9999999 * Math.random()).toString().padStart(7, '0') +
        Math.floor(9999999 * Math.random()).toString().padStart(7, '0')
      : Math.floor(99999 * Math.random()).toString().padStart(5, '0') +
        Math.floor(9999 * Math.random()).toString().padStart(4, '0')) + '00')
    return t + String(i - (o % i)).padStart(2, '0')
  }

  /**
   *
   * @param bban c 5 знака 16 символов
   * @return одну цифру
   */
  getEstonianCheckDigit (bban: string) {
    const toCheck = bban.substring(2, 15)
    const weights = [7, 1, 3, 7, 1, 3, 7, 1, 3, 7, 1, 3, 7]

    let sum = 0
    for (let index = 0; index < toCheck.length; index++) {
      sum += parseInt(toCheck.charAt(index), 10) * weights[index]
    }
    const remainder = sum % 10

    return remainder === 0 ? 0 : 10 - remainder
  };

  getBankCode (country: Country) {
    const plBankCodes = [
      '16000003', // (BNPPL Centrala) BNP PARIBAS BANK POLSKA SA Centrala
      '11602231', // Bank Millennium SA Bankowy Punkt Rozlicze Nr 4
      '10301612', // BH Regionalne Centrum Rozliczeń
      '18900002', // HYPO Centrala
      '15002240', // KBSA O. w Ełku
      '19401018', // CABP F. nr 3 we Wrocławiu
      '19101152'
    ]
    const eeBankCodes = [
      '2200', // Swedbank
      '1010', // SEB
      '3300', // Danske Bank A/S Eesti filiaal (Sampo Pank)
      '7700', // LHV Pank
      '1700', // Luminor
      '4204' // COOP Bank
    ]
    const ltBankCodes = ['20900', '72900', '74000', '40100', '21200', '72300', '21700', '21400', '70440', '71800', '73000']
    switch (country) {
      case Country.POLAND:
        return plBankCodes[Math.floor(Math.random() * plBankCodes.length)]
      case Country.ESTONIA:
        return eeBankCodes[Math.floor(Math.random() * eeBankCodes.length)]
      default:
        return ltBankCodes[Math.floor(Math.random() * ltBankCodes.length)]
    }
  }

  getIbanCheckDigits (e: string) {
    return String(BigInt('98') - (BigInt(e) % BigInt('97'))).padStart(2, '0')
  }
}

export default new IbanGenerator()
